@@ -4,7 +4,7 @@ use Digest::MD5 'md5';
 use Carp;
 use strict;
 use vars qw($VERSION);
-$VERSION = '2.24';
+$VERSION = '2.27';
 
 use constant RANDOM_DEVICE => '/dev/urandom';
 
@@ -116,24 +116,29 @@ sub new {
 	  unless ($rbs == $bs);
       }
     } else {
-      $padding = $padding eq 'null'          ? \&_null_padding
-	        :$padding eq 'space'         ? \&_space_padding
-		:$padding eq 'oneandzeroes'  ? \&_oneandzeroes_padding
-                :$padding eq 'standard'      ? \&_standard_padding
+      $padding = $padding eq 'null'           ? \&_null_padding
+	        :$padding eq 'space'          ? \&_space_padding
+		:$padding eq 'oneandzeroes'   ? \&_oneandzeroes_padding
+		:$padding eq 'rijndael_compat'? \&_rijndael_compat
+                :$padding eq 'standard'       ? \&_standard_padding
 	        :croak "'$padding' padding not supported.  See perldoc Crypt::CBC for instructions on creating your own.";
     }
 
     # CONSISTENCY CHECKS
     # HEADER consistency
     if ($header_mode eq 'salt') {
-      croak "Cannot use salt-based key generation if literal key is specified" if $options->{literal_key};
-      croak "Cannot use salt-based IV generation if literal IV is specified"   if exists $options->{iv};
+      croak "Cannot use salt-based key generation if literal key is specified"
+	if $options->{literal_key};
+      croak "Cannot use salt-based IV generation if literal IV is specified"
+	if exists $options->{iv};
     }
     elsif ($header_mode eq 'randomiv') {
-      croak "Cannot encrypt using a non-8 byte blocksize cipher when using randomiv header mode" unless $bs == 8 || $legacy_hack;
+      croak "Cannot encrypt using a non-8 byte blocksize cipher when using randomiv header mode"
+	unless $bs == 8 || $legacy_hack;
     }
     elsif ($header_mode eq 'none') {
-      croak "You must provide an initialization vector using -iv when using -header=>'none'" unless exists $options->{iv};
+      croak "You must provide an initialization vector using -iv when using -header=>'none'"
+	unless exists $options->{iv};
     }
 
     # KEYSIZE consistency
@@ -222,6 +227,10 @@ sub crypt (\$$){
 
     my $bs = $self->{'blocksize'};
 
+    croak "When using rijndael_compat padding, plaintext size must be a multiple of $bs"
+      if $self->{'padding'} eq \&_rijndael_compat
+	and length($data) % $bs;
+
     return $result unless (length($self->{'buffer'}) >= $bs);
 
     my @blocks = unpack("a$bs "x(int(length($self->{'buffer'})/$bs)) . "a*", $self->{'buffer'});
@@ -364,12 +373,34 @@ sub _generate_iv_and_cipher_from_options {
 
   croak "key and/or iv are missing" unless defined $self->{key} && defined $self->{civ};
 
+  $self->_taintcheck($self->{key});
   $self->{crypt} = ref $self->{cipher} ? $self->{cipher}
                                        : $self->{cipher}->new($self->{key})
 					 or croak "Could not create $self->{cipher} object: $@";
   return $result;
 }
 
+sub _taintcheck {
+    my $self = shift;
+    my $key  = shift;
+    return unless ${^TAINT};
+
+    my $has_scalar_util = eval "require Scalar::Util; 1";
+    my $tainted;
+
+    if ($has_scalar_util) {
+	$tainted = Scalar::Util::tainted($key);
+    } else {
+	local($@, $SIG{__DIE__}, $SIG{__WARN__});
+	local $^W = 0;
+	eval { kill 0 * $key };
+	$tainted = $@ =~ /^Insecure/;
+    }
+
+    croak "Taint checks are turned on and your key is tainted. Please untaint the key and try again"
+	if $tainted;
+}
+
 sub _key_from_key {
   my $self  = shift;
   my $pass  = shift;
@@ -468,7 +499,6 @@ sub _null_padding ($$$) {
 
 sub _oneandzeroes_padding ($$$) {
   my ($b,$bs,$decrypt) = @_;
-  return unless length $b;
   $b = length $b ? $b : '';
   if ($decrypt eq 'd') {
      my $hex = unpack("H*", $b);
@@ -478,6 +508,17 @@ sub _oneandzeroes_padding ($$$) {
   return $b . pack("C*", 128, (0) x ($bs - length($b) % $bs - 1) );
 }
 
+sub _rijndael_compat ($$$) {
+  my ($b,$bs,$decrypt) = @_;
+  return unless length $b;
+  if ($decrypt eq 'd') {
+     my $hex = unpack("H*", $b);
+     $hex =~ s/80*$//s;
+     return pack("H*", $hex);
+  }
+  return $b . pack("C*", 128, (0) x ($bs - length($b) % $bs - 1) );
+}
+
 sub get_initialization_vector (\$) {
   my $self = shift;
   $self->iv();
@@ -628,8 +669,9 @@ The new() method creates a new Crypt::CBC object. It accepts a list of
                     'randomiv' -- Randomiv-compatible "RandomIV" header
                     'none'   -- prepend no header at all
 
-  -padding        The padding method, one of "standard", "space",
-                     "onesandzeroes", or "null". (default "standard")
+  -padding        The padding method, one of "standard" (default),
+                     "space", "oneandzeroes", "rijndael_compat",
+                     or "null" (default "standard").
 
   -literal_key    If true, the key provided by "key" is used directly
                       for encryption/decryption.  Otherwise the actual
@@ -925,7 +967,7 @@ Use the 'padding' option to change the padding method.
 
 When the last block of plaintext is shorter than the block size,
 it must be padded. Padding methods include: "standard" (i.e., PKCS#5),
-"oneandzeroes", "space", and "null".
+"oneandzeroes", "space", "rijndael_compat" and "null".
 
    standard: (default) Binary safe
       pads with the number of bytes that should be truncated. So, if 
@@ -938,14 +980,21 @@ it must be padded. Padding methods include: "standard" (i.e., PKCS#5),
       block. If the last block is a full block and blocksize is 8, a
       block of "8000000000000000" will be appended.
 
+   rijndael_compat: Binary safe, with caveats
+      similar to oneandzeroes, except that no padding is performed if
+      the last block is a full block. This is provided for
+      compatibility with Crypt::Rijndael only and can only be used
+      with messages that are a multiple of the Rijndael blocksize
+      of 16 bytes.
+
    null: text only
       pads with as many "00" necessary to fill the block. If the last 
-      block is a full block and blocksize is 8, a block of 
+      block is a full block and blocksize is 8, a block of
       "0000000000000000" will be appended.
 
    space: text only
       same as "null", but with "20".
-      
+
 Both the standard and oneandzeroes paddings are binary safe.  The
 space and null paddings are recommended only for text data.  Which
 type of padding you use depends on whether you wish to communicate
